# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2012-2014 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.package.promise import limits
import os
import pkg_resources
import unittest

def _fake_call(self, *args, **kw):
  self.last_call = (args, kw)

class testLimitsTestCase(unittest.TestCase):

  def setUp(self):
    limits.Promise._call = _fake_call
    if os.path.exists("/tmp/test_promise_testing_limits.conf"):
      os.remove("/tmp/test_promise_testing_limits.conf")

  def testLimitsCheckConsistency(self):
    promise = limits.Promise()
    promise.configuration_file_path = "/tmp/test_promise_testing_limits.conf"

    self.assertFalse(promise.checkConsistency())

    open(promise.configuration_file_path, "w").write("# Something")

    self.assertFalse(promise.checkConsistency())

  def testLimitsFixConsistency(self):
    limits.Promise._call = _fake_call
    promise = limits.Promise()
    promise.configuration_file_path = "/tmp/test_promise_testing_limits.conf"

    self.assertFalse(promise.checkConsistency())
    self.assertTrue(promise.fixConsistency())
    self.assertTrue(promise.checkConsistency())

    self.assertTrue(os.path.exists(promise.configuration_file_path))
    limit_content = open(promise.configuration_file_path, "r").read()
    self.assertEquals(limit_content, pkg_resources.resource_stream(limits.__name__,
                                             'template/limits.conf.in').read())
